#!/usr/bin/env python3

# A script for building static blogs
# Copyright (C) 2020-2023 pood.re
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of version 3 of the GNU Affero General Public License
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# You probably should use an actually tested and well-made static blog engine
# instead. Just saying.
#
# Requirements:
# - BeautifulSoup (with lxml backend, although that can be modified)
# - GNU tar (maybe other tars work, untested)
#
# Tested on linux, *should* work on other platforms.
#
# Known flaws:
# - Data is not escaped to be HTML-safe. Should be OK as long as your metadata
#   isn't too wonky. Be careful.

import itertools
import io
import json
import os
import subprocess
import time

from bs4 import BeautifulSoup

start_time = time.monotonic()

current_dir = os.path.dirname(os.path.realpath(__file__))
os.chdir(current_dir)

def read_file(filename):
    with open(filename) as input_file:
        return input_file.read()

def read_json(filename):
    with open(filename) as input_file:
        return json.load(input_file)

format_options = read_json("info.json")
pages          = read_json("pages.json")

for file in os.listdir(os.path.join("output", "pages")):
    os.remove(os.path.join("output", "pages", file))

page_template    = read_file(os.path.join("templates", "page.html"))
listing_template = read_file(os.path.join("templates", "listing_entry.html"))
rss_template     = read_file(os.path.join("templates", "page_rss.xml"))
rss_listing      = read_file(os.path.join("templates", "listing_entry.xml"))

rss_page_buffer = io.StringIO()
listing_buffer  = io.StringIO()

print("Building articles...")

for page in pages:
    print("- {}".format(page["file"]))

    #article_contents = read_file()
    with open(os.path.join("pages", page["file"])) as input_file:
        article_contents = input_file.read()

    parsed_page = BeautifulSoup(article_contents, "lxml")

    article_format_options = format_options.copy()

    # The first <h2> is considered the article's title
    article_format_options["title"] = parsed_page.find("header").find("h2").get_text()
    # The first <p> is considered the article's description
    article_format_options["description"] = parsed_page.find("header").find("p").get_text()

    article_format_options["url"] = format_options["rel_url"] + "pages/" + page["file"]
    article_format_options["perma_url"] = format_options["base_url"] + "pages/" + page["file"]

    article_format_options["content"] = article_contents.format(
        **article_format_options, **page
    )

    listing_buffer.write(listing_template.format(
        **article_format_options,
        **page
    ))

    rss_page_buffer.write(rss_listing.format(
        **article_format_options,
        **page
    ))

    with open(os.path.join("output", "pages", page["file"]), "w") as output_file:
        output_file.write(page_template.format(**article_format_options, **page))

print("Building RSS feed...")
with open(os.path.join("output", "rss.xml"), "w") as rss_output:
    rss_output.write(rss_template.format(**format_options, content=rss_page_buffer.getvalue()))

print("Building index page...")
with open(os.path.join("output", "index.html"), "w") as listing_output:
    listing_output.write(page_template.format(**format_options, content=listing_buffer.getvalue()))

print("Building source page...")
with open(os.path.join("output", "source.html"), "w") as source_output:
    source_info = read_file(os.path.join("pages", "source.html")).format(**article_format_options)

    source_output.write(page_template.format(**format_options, content=source_info))

# Create an archive of the whole directory, excluding the archive itself.
print("Creating source archive...")

source_archive_path = os.path.join("output", "source.tar.gz")
subprocess.call([
    "tar",
    "--create",
    "--file", source_archive_path,
    "--exclude", source_archive_path,
    "--exclude", "wip",
    "."
])

print("Done in {:.3f}s".format(time.monotonic() - start_time))
